unit UDataModule_FB;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Phys.FBDef, FireDAC.Stan.Intf, FireDAC.Stan.Param,
  FireDAC.Phys.Intf, FireDAC.Stan.Option, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.DApt.Intf,
  FireDAC.Stan.Async, FireDAC.DApt, FireDAC.UI.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool,
  FireDAC.Phys, FireDAC.Phys.FB, FireDAC.VCLUI.Wait, FireDAC.Comp.UI, Data.DB, FireDAC.Comp.Client,
  FireDAC.Comp.DataSet, FireDAC.Phys.IBBase, System.JSON;

type
  T_DataModule_FB = class(TDataModule)
    FDPhysFBDriverLink1: TFDPhysFBDriverLink;
    FDConnection_Jokusou: TFDConnection;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDQuery_PatientInfo: TFDQuery;
    FDQuery_PatientInfoFURIGANA: TWideStringField;
    FDQuery_PatientInfoNAMAE: TWideStringField;
    FDQuery_PatientInfoSEX: TSmallintField;
    FDQuery_PatientInfoB_DAY: TDateField;
    FDConnection_Patient: TFDConnection;
    FDQuery_BedsoBunruiList: TFDQuery;
    FDQuery_BedsoBunruiListPART_GROUP_ID: TIntegerField;
    FDQuery_BedsoBunruiListPART_GROUP: TWideStringField;
    FDQuery_WardKubunList: TFDQuery;
    FDQuery_WardKubunListBYOUTOU_NAME: TWideStringField;
    FDQuery_BedsoreKubunList: TFDQuery;
    FDQuery_BedsoreKubunListPART_NAME: TWideStringField;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private 錾 }
    var
      OutputPath: string;
  public
    { Public 錾 }
    function NenreiCalk(BDay: TDateTime): integer;
    function GetPatientInfo(PATIENT_ID: integer): TJSONObject;
    function GetWardKubunList: TJSONArray;
    function GetBedsoreBunruiList: TJSONArray;
    function GetBedsoreKubunList(Bunrui: Integer): TJSONArray;
  end;

var
  _DataModule_FB: T_DataModule_FB;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

uses IniFiles, Vcl.Forms, Vcl.Dialogs, Winapi.Windows, UBedsoreMainForm;

{$R *.dfm}

procedure T_DataModule_FB.DataModuleCreate(Sender: TObject);
var
  ini: TIniFile;
  Path, FB_Client, USERNAME, PASSWD,
  PatientDBPath, PatientDBName,
  PictureDBPath, PictureDBName: string;
begin
  SetLength(Path, MAX_PATH);
  Path := ExtractFilePath(Application.ExeName);
  Ini:=TIniFile.Create(Path + 'Jokusou.ini');

  FDConnection_Patient.Connected := false;

  try
    PatientDBPath := Ini.ReadString('Database', 'PatientHostName', '');
    PatientDBName := Ini.ReadString('Database', 'PatientDBName', '');
    PictureDBPath := Ini.ReadString('Database', 'PictureHostName', '');
    PictureDBName := Ini.ReadString('Database', 'PictureDBName', '');
    FB_Client := Ini.ReadString('Database', 'FB_Client', '');
    USERNAME := Ini.ReadString('Database', 'USER_NAME', '');
    PASSWD := Ini.ReadString('Database', 'PASSWD', '');
    OutputPath := Ini.ReadString('Picture', 'OutputPath', '');
  finally
    Ini.Free;
  end;

  FDPhysFBDriverLink1.VendorLib := FB_Client;

  FDConnection_Patient.Params.Clear;
  FDConnection_Patient.Params.Add('Database=' + PatientDBName);
  FDConnection_Patient.Params.Add('User_Name=' + USERNAME);
  FDConnection_Patient.Params.Add('Password=' + PASSWD);
  FDConnection_Patient.Params.Add('RoleName=H_HP');
  FDConnection_Patient.Params.Add('Protocol=TCPIP');
  FDConnection_Patient.Params.Add('Server=' + PatientDBPath);
  FDConnection_Patient.Params.Add('CharacterSet=UTF8');
  FDConnection_Patient.Params.Add('DriverID=PG');

  try
    FDConnection_Patient.Connected := true;
  except
    ShowMessage('Patient Database Connect Error');
    _BedsoreMainForm.Close;
  end;

  FDConnection_Jokusou.Params.Clear;
  FDConnection_Jokusou.Params.Add('Database=' + PictureDBName);
  FDConnection_Jokusou.Params.Add('User_Name=' + USERNAME);
  FDConnection_Jokusou.Params.Add('Password=' + PASSWD);
  FDConnection_Jokusou.Params.Add('RoleName=H_HP');
  FDConnection_Jokusou.Params.Add('Protocol=TCPIP');
  FDConnection_Jokusou.Params.Add('Server=' + PictureDBPath);
  FDConnection_Jokusou.Params.Add('CharacterSet=UTF8');
  FDConnection_Jokusou.Params.Add('DriverID=FB');

  try
    FDConnection_Jokusou.Connected := true;
  except
    ShowMessage('Jokusou Database Connect Error');
    _BedsoreMainForm.Close;
  end;

  _BedsoreMainForm.lServerPath1.Caption := PictureDBPath;
  _BedsoreMainForm.lDatabaseName1.Caption := PictureDBName;
  _BedsoreMainForm.lServerPath2.Caption := PatientDBPath;
  _BedsoreMainForm.lDatabaseName2.Caption := PatientDBName;
  _BedsoreMainForm.lSavePath.Caption := OutputPath;
end;

procedure T_DataModule_FB.DataModuleDestroy(Sender: TObject);
begin
  FDConnection_Patient.Connected := false;
  FDConnection_Jokusou.Connected := false;
end;

function T_DataModule_FB.GetBedsoreBunruiList: TJSONArray;
var
  qr: TFDQuery;
  Obj: TJSONObject;
  Arry: TJSONArray;
begin
  //Bedso Kubun List
  qr := FDQuery_BedsoBunruiList;
  qr.ResourceOptions.CmdExecMode := amBlocking;
  Arry := TJSONArray.Create;

  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;
      Obj.AddPair('Bunrui', qr.FieldByName('PART_GROUP').AsWideString);
      Arry.AddElement(Obj);
      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_FB.GetBedsoreKubunList(Bunrui: Integer): TJSONArray;
var
  qr: TFDQuery;
  Obj: TJSONObject;
  Arry: TJSONArray;
begin
  //ጋ敪
  qr := FDQuery_BedsoreKubunList;
  Arry := TJSONArray.Create;

  qr.ParamByName('GROUP_ID').AsInteger := Bunrui;
  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;
      Obj.AddPair('Kubun', qr.FieldByName('PART_NAME').AsWideString);
      Arry.AddElement(Obj);
      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_FB.GetPatientInfo(PATIENT_ID: integer): TJSONObject;
var
  qr: TFDQuery;
  Obj: TJSONObject;
  rslt: TJSONObject;
begin
  //Patient Info
  qr := FDQuery_PatientInfo;
  qr.ResourceOptions.CmdExecMode := amBlocking;
  Obj := TJSONObject.Create;

  qr.ParamByName('KANJA_ID').AsInteger := Patient_ID;
  try
    qr.Open();
    if not(qr.Eof) then
    begin
      Obj.AddPair('PATIENT_ID', IntToStr(Patient_ID));
      Obj.AddPair('FURIGANA', qr.FieldByName('FURIGANA').AsString);
      Obj.AddPair('NAMAE', qr.FieldByName('NAMAE').AsString);
      Obj.AddPair('BDAY', FormatDateTime('ggeN m d', qr.FieldByName('B_DAY').AsDateTime) +
                          '(' + IntToStr(NenreiCalk(qr.FieldByName('B_DAY').AsDateTime)) + ')');
      if qr.FieldByName('SEX').AsInteger = 1 then
        Obj.AddPair('SEX', 'j')
      else
        Obj.AddPair('SEX', '');
    end else
    begin
      Obj.AddPair('PATIENT_ID', '0');
      Obj.AddPair('FURIGANA', '');
      Obj.AddPair('NAMAE', '');
      Obj.AddPair('SEX', '');
    end;
  finally
    qr.Close;
  end;

  rslt := TJSONObject.Create;
  rslt.AddPair('Result', Obj);

  Result := rslt;
end;

function T_DataModule_FB.GetWardKubunList: TJSONArray;
var
  qr: TFDQuery;
  Arry: TJSONArray;
  Obj: TJSONObject;
begin
  qr := FDQuery_WardKubunList;
  qr.ResourceOptions.CmdExecMode := amBlocking;
  Arry := TJSONArray.Create;

  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;
      Obj.AddPair('WardKubun', qr.FieldByName('BYOUTOU_NAME').AsString);
      Arry.AddElement(Obj);
      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_FB.NenreiCalk(BDay: TDateTime): integer;
var
  Year1, Year2, Month1, Day1, Month2, Day2: word;
  Nenrei: integer;
begin
  DecodeDate(Now, Year1, Month1, Day1);
  DecodeDate(BDay, Year2, Month2, Day2);
  Nenrei := Year1 - Year2;
  if Month1 = Month2 then begin
    if Day1 < Day2 then
      Nenrei := Nenrei - 1;
  end else if Month1 < Month2 then begin
    Nenrei := Nenrei - 1;
  end;
  result := Nenrei;
end;

end.
