unit UDataModule_PG;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, Vcl.Graphics,
  FireDAC.Stan.Option, FireDAC.Stan.Error,
  FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def, FireDAC.Stan.Pool, FireDAC.Stan.Async,
  FireDAC.Phys, FireDAC.VCLUI.Wait, Data.DB, FireDAC.Comp.Client, FireDAC.Phys.PG,
  FireDAC.Phys.PGDef, FireDAC.Stan.Param, FireDAC.DatS, FireDAC.DApt.Intf, FireDAC.DApt,
  FireDAC.Comp.UI, FireDAC.Comp.DataSet, System.JSON, FireDAC.Moni.Base,
  FireDAC.Moni.RemoteClient, Vcl.Forms, JPEG, EncdDecd, System.NetEncoding;

type
  T_DataModule_PG = class(TDataModule)
    FDConnection_BedSore: TFDConnection;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDQuery_WardKubun: TFDQuery;
    FDQuery_WardKubunWardKubun: TWideStringField;
    FDPhysPgDriverLink1: TFDPhysPgDriverLink;
    FDQuery_BedsoreBunruiList: TFDQuery;
    FDQuery_BedsoreKubunList: TFDQuery;
    FDQuery_BedsoreBunruiListPartGroup: TWideStringField;
    FDQuery_BedsoreKubunListPartName: TWideStringField;
    FDConnection_PatientDB: TFDConnection;
    FDQuery_PatientInfo: TFDQuery;
    FDQuery_PatientInfoFurigana: TWideStringField;
    FDQuery_PatientInfoNamae: TWideStringField;
    FDQuery_PatientInfoGender: TIntegerField;
    FDQuery_PatientInfoBDay: TDateField;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private 錾 }
    var
      OutputPath: string;
    function NenreiCalk(BDay: TDateTime): integer;
  public
    { Public 錾 }
    function GetPatientInfo(PATIENT_ID: integer): TJSONObject;
    function GetWardKubunList: TJSONArray;
    function GetBedsoreBunruiList: TJSONArray;
    function GetBedsoreKubunList(Bunrui: Integer): TJSONArray;
    function PictureTransfer(Obj: TJSONObject): Boolean;
  end;

var
  _DataModule_PG: T_DataModule_PG;

implementation

{%CLASSGROUP 'Vcl.Controls.TControl'}

uses IniFiles, Vcl.Dialogs, Winapi.Windows, UBedsoreMainForm;

{$R *.dfm}

{ TDataModule_PG }

procedure T_DataModule_PG.DataModuleCreate(Sender: TObject);
var
  ini: TIniFile;
  Path, PG_Client, USERNAME, PASSWD,
  PatientDBServer, PatientDBName,
  BedsoreDBServer, BedsoreDBName: string;
begin
  SetLength(Path, MAX_PATH);
  Path := ExtractFilePath(Application.ExeName);
  Ini:=TIniFile.Create(Path + 'Jokusou.ini');

  try
    PatientDBServer := Ini.ReadString('Database', 'PatientDBServer', '');
    PatientDBName := Ini.ReadString('Database', 'PatientDBName', '');
    BedsoreDBServer := Ini.ReadString('Database', 'BedsoreDBServer', '');
    BedsoreDBName := Ini.ReadString('Database', 'BedsoreDBName', '');
    PG_Client := Ini.ReadString('Database', 'PG_Client', '');
    USERNAME := Ini.ReadString('Database', 'USER_NAME', '');
    PASSWD := Ini.ReadString('Database', 'PASSWD', '');
    OutputPath := Ini.ReadString('Picture', 'OutputPath', '');
  finally
    Ini.Free;
  end;

  FDConnection_BedSore.Connected := false;
  FDConnection_BedSore.Params.Clear;
  FDConnection_BedSore.Params.Add('Database=' + BedsoreDBName);
  FDConnection_BedSore.Params.Add('User_Name=' + USERNAME);
  FDConnection_BedSore.Params.Add('Password=' + PASSWD);
  FDConnection_BedSore.Params.Add('Server=' + BedsoreDBServer);
  FDConnection_BedSore.Params.Add('CharacterSet=UTF8');
  FDConnection_BedSore.Params.Add('ExtendedMetadata=True');
  FDConnection_BedSore.Params.Add('DriverID=PG');
  FDPhysPgDriverLink1.VendorLib := PG_Client;
  FDConnection_BedSore.Connected := true;

  FDConnection_PatientDB.Connected := false;
  FDConnection_PatientDB.Params.Clear;
  FDConnection_PatientDB.Params.Add('Database=' + PatientDBName);
  FDConnection_PatientDB.Params.Add('User_Name=' + USERNAME);
  FDConnection_PatientDB.Params.Add('Password=' + PASSWD);
  FDConnection_PatientDB.Params.Add('Server=' + PatientDBServer);
  FDConnection_PatientDB.Params.Add('CharacterSet=UTF8');
  FDConnection_PatientDB.Params.Add('ExtendedMetadata=True');
  FDConnection_PatientDB.Params.Add('DriverID=PG');
  FDConnection_PatientDB.Connected := true;

  _BedsoreMainForm.lServerPath1.Caption := BedsoreDBServer;
  _BedsoreMainForm.lDatabaseName1.Caption := BedsoreDBName;
  _BedsoreMainForm.lServerPath2.Caption := PatientDBServer;
  _BedsoreMainForm.lDatabaseName2.Caption := PatientDBName;
  _BedsoreMainForm.lSavePath.Caption := OutputPath;
end;

function T_DataModule_PG.GetBedsoreBunruiList: TJSONArray;
var
  qr: TFDQuery;
  Arry: TJSONArray;
  Obj: TJSONObject;
begin
  qr := FDQuery_BedsoreBunruiList;
  Arry := TJSONArray.Create;

  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;

      Obj.AddPair('Bunrui', qr.FieldByName('PartGroup').AsWideString);

      Arry.AddElement(Obj);

      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_PG.GetBedsoreKubunList(Bunrui: Integer): TJSONArray;
var
  qr: TFDQuery;
  Arry: TJSONArray;
  Obj: TJSONObject;
begin
  qr := FDQuery_BedsoreKubunList;
  Arry := TJSONArray.Create;

  qr.ParamByName('GroupID').AsInteger := Bunrui;
  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;

      Obj.AddPair('Kubun', qr.FieldByName('PartName').AsWideString);

      Arry.AddElement(Obj);

      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_PG.GetPatientInfo(PATIENT_ID: integer): TJSONObject;
var
  qr: TFDQuery;
  Obj, rslt: TJSONObject;
begin
  qr := FDQuery_PatientInfo;
  Obj := TJSONObject.Create;
  rslt := TJSONObject.Create;

  qr.ParamByName('Patient_ID').AsInteger := PATIENT_ID;
  try
    qr.Open();
    if not(qr.Eof) then
    begin
      Obj.AddPair('PATIENT_ID', IntToStr(Patient_ID));
      Obj.AddPair('Furigana', qr.FieldByName('FURIGANA').AsString);
      Obj.AddPair('Name', qr.FieldByName('NAMAE').AsString);
      Obj.AddPair('BDay', FormatDateTime('ggeN m d', qr.FieldByName('BDAY').AsDateTime) +
                          '(' + IntToStr(NenreiCalk(qr.FieldByName('BDAY').AsDateTime)) + ')');
      if qr.FieldByName('Gender').AsInteger = 1 then
        Obj.AddPair('Gender', 'j')
      else
        Obj.AddPair('Gender', '');
    end else
    begin
      Obj.AddPair('PATIENT_ID', '0');
      Obj.AddPair('Furigana', '');
      Obj.AddPair('Name', '');
      Obj.AddPair('Gender', '');
    end;
   finally
    qr.Close;
  end;

  rslt.AddPair('Result', obj);
  Result := rslt;
end;

function T_DataModule_PG.GetWardKubunList: TJSONArray;
var
  qr: TFDQuery;
  Arry: TJSONArray;
  Obj: TJSONObject;
begin
  //a敪
  qr := FDQuery_WardKubun;
  Arry := TJSONArray.Create;

  try
    qr.Open();
    while not(qr.Eof) do
    begin
      Obj := TJSONObject.Create;

      Obj.AddPair('WardKubun', qr.FieldByName('WardKubun').AsString);

      Arry.AddElement(Obj);

      qr.Next;
    end;
  finally
    qr.Close;
  end;

  Result := Arry;
end;

function T_DataModule_PG.NenreiCalk(BDay: TDateTime): integer;
var
  Year1, Year2, Month1, Day1, Month2, Day2: word;
  Nenrei: integer;
begin
  DecodeDate(Now, Year1, Month1, Day1);
  DecodeDate(BDay, Year2, Month2, Day2);
  Nenrei := Year1 - Year2;
  if Month1 = Month2 then
  begin
    if Day1 < Day2 then
      Nenrei := Nenrei - 1;
  end else if Month1 < Month2 then
  begin
    Nenrei := Nenrei - 1;
  end;
  result := Nenrei;
end;

function T_DataModule_PG.PictureTransfer(Obj: TJSONObject): Boolean;
var
  lp: Integer;
  WardKubun, Bunrui, Kubun, FileName: string;
  DataPath, Path, buf, buf2: string;
  PatientID: integer;
  Study_Date: TDateTime;
  smPicture: AnsiString;
  Stream: TMemoryStream;
  byt: TBytes;
  jpg, jpg2, jpg3: TJPEGImage;
  bmp, bmp2: Vcl.Graphics.TBitmap;
  flg: Boolean;
begin
  jpg := TJPEGImage.Create;
  jpg2 := TJPEGImage.Create;
  jpg3 := TJPEGImage.Create;
  Stream := TMemoryStream.Create;
  PatientID := 0;
  Study_Date := Now;
  flg := false;

  _BedsoreMainForm.lPatientID.Caption := '';
  _BedsoreMainForm.lSatueiBunrui.Caption := '';
  _BedsoreMainForm.lSatueiKubun.Caption := '';
  _BedsoreMainForm.lName.Caption := '';
  _BedsoreMainForm.lPictureName.Caption := '';

  for lp := 0 to Obj.Count - 1 do
  begin
    if Obj.Pairs[lp].JsonString.Value = 'PATIENT_ID' then
      PatientID := StrToIntDef(Obj.Get('PATIENT_ID').JsonValue.Value, 0)
    else if Obj.Pairs[lp].JsonString.Value = 'STUDY_DATE' then
      Study_Date := StrToDateTime(Obj.Get('STUDY_DATE').JsonValue.Value)
    else if Obj.Pairs[lp].JsonString.Value = 'BYOUTOU_NAME' then
      WardKubun := Obj.Get('BYOUTOU_NAME').JsonValue.Value
    else if Obj.Pairs[lp].JsonString.Value = 'DECUBITUS_BUNRUI' then
      Bunrui := Obj.Get('DECUBITUS_BUNRUI').JsonValue.Value
    else if Obj.Pairs[lp].JsonString.Value = 'DECUBITUS_KUBUN' then
      Kubun  := Obj.Get('DECUBITUS_KUBUN').JsonValue.Value
    else if Obj.Pairs[lp].JsonString.Value = 'FILE_NAME' then
      FileName := Obj.Get('FILE_NAME').JsonValue.Value
    else if Obj.Pairs[lp].JsonString.Value = 'DECUBITUS_PICTURE' then
      smPicture := AnsiString(Obj.Get('DECUBITUS_PICTURE').JsonValue.Value);
  end;

  _BedsoreMainForm.lPatientID.Caption := FormatFloat('000000', PatientID);
  _BedsoreMainForm.lSatueiBunrui.Caption := Bunrui;
  _BedsoreMainForm.lSatueiKubun.Caption := Kubun;
  _BedsoreMainForm.lName.Caption := '';
  _BedsoreMainForm.lPictureName.Caption := FileName;

  //摜\
  byt := DecodeBase64(smPicture);

  Stream.Write(byt[0], Length(byt));

  //摜o͐쐬
  DataPath := '\ID' + FormatFloat('00000000', PatientID) +
              '\RS' + FormatDateTime('yyyymm', Study_Date) +
              '\' + FormatDateTime('dd', Study_Date);
  Path := OutputPath + DataPath;

  if not DirectoryExists(Path) then
  begin
    buf := Path; buf2 := '';
    while AnsiPos('\', buf) > 0 do
    begin
      buf2 := buf2 + Copy(buf, 1, AnsiPos('\', buf));
      if not DirectoryExists(buf2) then MkDir(buf2);
      buf := Copy(buf, AnsiPos('\', buf) + 1, Length(buf));
    end;
    if not DirectoryExists(Path) then MkDir(Path);
  end;

  //TlCGAP̍쐬
  buf := Path + '\SamNail'; buf2:='';
  while AnsiPos('\',buf) > 0 do
  begin
      buf2 := buf2 + Copy(buf, 1, AnsiPos('\', buf));
      if not DirectoryExists(buf2) then MkDir(buf2);
      buf := Copy(buf, AnsiPos('\', buf) + 1, Length(buf));
  end;
  if not(DirectoryExists(Path + '\SamNail')) then ForceDirectories(Path + '\SamNail');

  //TlCGAQ̍쐬
  buf := Path + '\SamNail2'; buf2:='';
  while AnsiPos('\',buf) > 0 do
  begin
      buf2 := buf2 + Copy(buf, 1, AnsiPos('\', buf));
      if not DirectoryExists(buf2) then MkDir(buf2);
      buf := Copy(buf, AnsiPos('\', buf) + 1, Length(buf));
  end;
  if not(DirectoryExists(Path + '\SamNail2')) then ForceDirectories(Path + '\SamNail2');

  Stream.Position := 0;
  try
    jpg.LoadFromStream(Stream);
    _BedsoreMainForm.Image1.Picture.Assign(jpg);
    //Picture Save
    jpg.SaveToFile(Path + '\' + FileName);
    //TlCP쐬
    //TCYύX
    bmp := Vcl.Graphics.TBitmap.Create;
    bmp2 := Vcl.Graphics.TBitmap.Create;
    bmp.Assign(jpg);
    bmp2.Width := bmp.Width div 4;
    bmp2.Height := bmp.Height div 4;
    bmp2.Canvas.StretchDraw(RECT(0, 0, bmp2.Width - 1, bmp2.Height - 1), bmp);
    jpg2.Assign(bmp2);
    jpg2.SaveToFile(Path + '\SamNail\' + FileName);
    bmp.Free;
    bmp2.Free;
    //TlCQ쐬
    //TCYύX
    bmp := Vcl.Graphics.TBitmap.Create;
    bmp2 := Vcl.Graphics.TBitmap.Create;
    bmp.Assign(jpg);
    bmp2.Width := bmp.Width div 2;
    bmp2.Height := bmp.Height div 2;
    bmp2.Canvas.StretchDraw(RECT(0,0,bmp2.Width-1,bmp2.Height-1),bmp);
    jpg2.Assign(bmp2);
    jpg2.SaveToFile(Path + '\SamNail2\' + FileName);
    bmp.Free;
    bmp2.Free;
    flg := true;
  except on E: Exception do
    ShowMessage(E.Message);
  end;

  jpg.Free;
  jpg2.Free;
  jpg3.Free;

  Result := flg;
end;

end.
